#include <stm32l0xx.h>
#include <stdint.h>
#include "usb.h"

volatile unsigned int i = 0x12345678;
unsigned char c = 2;
unsigned int my_static_var;

volatile uint32_t tick = 0;

/*
 * Diese Funktion wird an sich nicht benötigt. usb_init und usb_enable führen diese Schritte aus.
 */
static void setup_usb_clock(void)
{
	uint32_t reload_val;
	const uint32_t felim = 0x22;
	const uint32_t f_target = 48000000UL;
	const uint32_t f_usb_sof = 1000UL;

	RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN;
	/* Enable the internal voltage reference. This is needed for HSI48 */
	SYSCFG->CFGR3 |= SYSCFG_CFGR3_EN_VREFINT;
	while (!(SYSCFG->CFGR3 & SYSCFG_CFGR3_VREFINT_RDYF));
	SYSCFG->CFGR3 |= SYSCFG_CFGR3_ENREF_HSI48;
	
	RCC->CRRCR = RCC_CRRCR_HSI48ON;
	while(!(RCC->CRRCR & RCC_CRRCR_HSI48RDY));
	RCC->CCIPR |= RCC_CCIPR_HSI48SEL;

	/* Configure Clock recovery from USB SOF (1kHz) */
	RCC->APB1ENR |= RCC_APB1ENR_CRSEN;
	reload_val = (f_target / f_usb_sof) - 1;
	CRS->CFGR = (reload_val & 0xFFFF) | (felim << 16) | CRS_CFGR_SYNCSRC_1;
	//CRS->CR = CRS_CR_CEN | CRS_CR_AUTOTRIMEN;
}

int main(void)
{
	RCC->IOPENR |= RCC_IOPENR_IOPAEN;
	GPIOA->MODER &= ~(3<<(0*2)) & ~(3<<(1*2)) & ~(3<<(2*2)) & ~(3<<(3*2));
	GPIOA->MODER |= (1<<(0*2)) | (1<<(1*2)) | (1<<(2*2)) | (1<<(3*2));
	GPIOA->ODR |= (1<<0) | (1<<2);
	SysTick_Config(3200000);
//setup_usb_clock();
    __enable_irq();

	usb_init();
	tick = 0;
	while (tick < 12);
	usb_enable();
	
	while (1);
}

void SysTick_Handler(void)
{
	tick++;
}
